@@ -1,6 +1,6 @@
 //===========================================================================
 // @(#) $Name$
-// @(#) $Id: DwmDirectoryEntry.cc 8388 2016-04-17 03:23:45Z dwm $
+// @(#) $Id: DwmDirectoryEntry.cc 9671 2017-06-28 07:35:24Z dwm $
 //===========================================================================
 //  Copyright (c) Daniel W. McRobb 2011
 //  All rights reserved.
@@ -57,12 +57,14 @@ extern "C" {
 #include "DwmSysLogger.hh"
 #include "DwmDirectoryEntry.hh"
 
-static const Dwm::SvnTag svntag("@(#) $DwmPath: dwm/libDwm/tags/libDwm-0.6.10/src/DwmDirectoryEntry.cc 8388 $");
+static const Dwm::SvnTag svntag("@(#) $DwmPath: dwm/libDwm/tags/libDwm-0.6.11/src/DwmDirectoryEntry.cc 9671 $");
 
 using namespace std;
 
 namespace Dwm {
 
+  std::mutex  DirectoryEntry::_readdirMtx;
+  
   //--------------------------------------------------------------------------
   //!  
   //--------------------------------------------------------------------------
@@ -224,20 +226,24 @@ namespace Dwm {
     if (IsDirectory()) {
       DIR  *dir = opendir(_path.c_str());
       if (dir) {
-        struct dirent   dirEnt;
-        struct dirent  *dirEnts = 0;
-        
-        while ((readdir_r(dir, &dirEnt, &dirEnts)) == 0
-               && dirEnts == &dirEnt) {
-          if (string(dirEnt.d_name) != "."
-              && string(dirEnt.d_name) != "..") {
-            if (GetType(dirEnt) & typeMask) {
-              children.push_back(DirectoryEntry(_path 
-                                                + string("/") 
-                                                + string(dirEnt.d_name), 
-                                                this));
+        struct dirent  *dirEnt;
+        bool            done = false;
+        while (! done) {
+          lock_guard<std::mutex>  readdirLock(_readdirMtx);
+          if ((dirEnt = readdir(dir)) != nullptr) {
+            if (string(dirEnt->d_name) != "."
+              && string(dirEnt->d_name) != "..") {
+              if (GetType(*dirEnt) & typeMask) {
+                children.push_back(DirectoryEntry(_path 
+                                                  + string("/") 
+                                                  + string(dirEnt->d_name), 
+                                                  this));
+              }
             }
           }
+          else {
+            done = true;
+          }
         }
         closedir(dir);
       }
@@ -258,18 +264,24 @@ namespace Dwm {
     if (IsDirectory()) {
       DIR  *dir = opendir(_path.c_str());
       if (dir) {
-        struct dirent   dirEnt;
-        struct dirent  *dirEnts = 0;
-        while ((readdir_r(dir, &dirEnt, &dirEnts) == 0)
-               && (dirEnts == &dirEnt)) {
-          if (string(dirEnt.d_name) != "."
-              && string(dirEnt.d_name) != "..") {
-            DirectoryEntry  de(_path + string("/") + string(dirEnt.d_name),
-                               this);
-            if (dirEntryFunc(de)) {
-              children.push_back(de);
+        struct dirent  *dirEnt;
+        bool            done = false;
+        while (! done) {
+          lock_guard<std::mutex>  readdirLock(_readdirMtx);
+          dirEnt = readdir(dir);
+          if (dirEnt != nullptr) {
+            if (string(dirEnt->d_name) != "."
+                && string(dirEnt->d_name) != "..") {
+              DirectoryEntry  de(_path + string("/") + string(dirEnt->d_name),
+                                 this);
+              if (dirEntryFunc(de)) {
+                children.push_back(de);
+              }
             }
           }
+          else {
+            done = true;
+          }
         }
         closedir(dir);
       }
